﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Xml;
using Microsoft.Win32;
using System.IO;
using System.Net.NetworkInformation; //for ping

namespace ProxySwap
{
    public partial class ProxySwap_Window : Form
    {
        #region Variables
        string XMLPath = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\ProxySwap";
        string XMLFile = Environment.GetFolderPath(Environment.SpecialFolder.ApplicationData) + @"\ProxySwap\ProxySwap.xml";   //@"C:\Users\Default\AppData\Roaming\ProxySwap\ProxySwap.xml";
        ListViewItem lvItem;
        public string newComment;
        public string pulledComment; 
        #endregion

        public ProxySwap_Window()
        {
            InitializeComponent();
            Get_Proxies();
            GetProxyFromRegistry();
        }

        #region Create XML File
        private void CreateXMLFile()
        {
            System.IO.Directory.CreateDirectory(XMLPath);
            FileStream fs = File.Create(XMLFile);
            fs.Dispose();
        }
        #endregion

        #region Get Proxy From Registry
        private void GetProxyFromRegistry()
        {
            RegistryKey SystemProxy = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Internet Settings");
            try
            {
                string UsedProxy = SystemProxy.GetValue("ProxyServer").ToString();
                foreach (ListViewItem item in listView1.Items)
                {
                    if (UsedProxy == item.Text + ":" + item.SubItems[1].Text)
                    {
                        item.Checked = true;
                    }
                    else
                    {
                        item.Checked = false;
                    }
                }
            }
            catch (Exception) 
            {
            }
        }
        #endregion

        #region Get Proxies From XML To ListView
        private void Get_Proxies()
        {
            listView1.Items.Clear();
            XmlDocument ProxyList = new System.Xml.XmlDocument();
            try
            {
                ProxyList.Load(XMLFile);
            }
            catch (Exception) 
            {
            }

            foreach (XmlNode IPNode in ProxyList.SelectNodes("/ProxyList/IP"))
            {
                lvItem = listView1.Items.Add(IPNode.InnerText);
                lvItem.SubItems.Add(new ListViewItem.ListViewSubItem(lvItem, IPNode.Attributes["port"].InnerText)); ;
                lvItem.SubItems.Add(new ListViewItem.ListViewSubItem(lvItem, IPNode.Attributes["country"].InnerText)); ;
                lvItem.SubItems.Add(""); ;  // --- delay  --- // --- needed for creating a subitem so I could change it's text by index in Pinging Proxy
                lvItem.SubItems.Add(""); ;  // --- status --- //
                lvItem.SubItems.Add(new ListViewItem.ListViewSubItem(lvItem, IPNode.Attributes["comment"].InnerText)); ;

            }            
        }
        #endregion

        #region Add Proxies To ListView
        private void AddProxiesToListView()
        {
            if (string.IsNullOrEmpty(Proxy_textBox.Text) || string.IsNullOrEmpty(Port_textBox.Text))
            {
                MessageBox.Show("Proxy and/or Port fields cannot be left empty!", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
            else
            {
                ListViewItem lvItem = new ListViewItem();
                lvItem.Text = Proxy_textBox.Text;
                lvItem.SubItems.Add(Port_textBox.Text);
                lvItem.SubItems.Add(Country_textBox.Text);
                lvItem.SubItems.Add("");   // --- delay  --- // --- required for subitems being created, so i can change their text by index in Pinging Proxy region
                lvItem.SubItems.Add("");   // --- status --- //
                lvItem.SubItems.Add(Comment_textBox.Text);
                listView1.Items.Add(lvItem);

                Proxy_textBox.Text = "";
                Port_textBox.Text = "";
                Country_textBox.Text = "";
                Comment_textBox.Text = "";
            }

            ///////
            
        }
#endregion

        #region Remove Individual Proxies From ListView
        private void DeleteItem()
        {
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                item.Remove();
                WriteListViewToXML();
            }
        }
        #endregion

        #region Write ListView Items To XML File
        private void WriteListViewToXML()
        {
            XmlTextWriter writer = new System.Xml.XmlTextWriter(XMLFile, null);

            writer.WriteStartElement("ProxyList");

            for (int i = 0; i < listView1.Items.Count; i++)
            {
                writer.WriteStartElement("IP");
                writer.WriteAttributeString("port", listView1.Items[i].SubItems[1].Text); ;
                writer.WriteAttributeString("country", listView1.Items[i].SubItems[2].Text); ;
                writer.WriteAttributeString("comment", listView1.Items[i].SubItems[5].Text); ;
                writer.WriteValue(listView1.Items[i].SubItems[0].Text);
                writer.WriteEndElement();
            }
            writer.WriteEndElement();
            writer.Close();

        }
        #endregion

        #region Allowing To Check Only One Item In ListView
        void listView1_ItemCheck(object sender, ItemCheckEventArgs e)
        {
            if (e.CurrentValue == CheckState.Checked)
                return;
            foreach (ListViewItem item in listView1.Items)
            {
                if (item.Index != e.Index)
                {
                    item.Checked = false;
                }
            }
        }

        #endregion

        #region Pinging Proxy
        private void Ping_Current_Proxy() //--- pinging checeked proxy on pressing Apply button ---//
        {
            foreach (ListViewItem item in listView1.CheckedItems)
            {
                try
                {
                    Ping pingIP = new Ping();
                    PingReply pingReply = pingIP.Send(item.Text);
                    if (pingReply.Status == IPStatus.Success)
                    {
                        item.SubItems[3].Text = pingReply.RoundtripTime.ToString();
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Green;
                    }
                    else
                    {
                        item.SubItems[3].Text = "";
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Red;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Could not ping remote computer! \nMake sure you have internet connection and you've typed IP address correctly! \n[XXX.XXX.XXX.XXX]", "Error pinging selected IP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    item.SubItems[3].Text = "";
                    item.SubItems[4].Text = "Failed";
                    item.UseItemStyleForSubItems = false;
                    item.SubItems[4].ForeColor = Color.Red;
                }
            }
        }

        private void Ping_Selected_Proxy() //--- pinging selected proxy via context menu ---//
        {
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                try
                {
                    Ping pingIP = new Ping();
                    PingReply pingReply = pingIP.Send(item.Text);
                    if (pingReply.Status == IPStatus.Success)
                    {
                        item.SubItems[3].Text = pingReply.RoundtripTime.ToString();
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Green;
                    }
                    else
                    {
                        item.SubItems[3].Text = "";
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Red;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Could not ping remote computer! \nMake sure you have internet connection and you've typed IP address correctly! \n[XXX.XXX.XXX.XXX]", "Error pinging selected IP", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    item.SubItems[3].Text = "";
                    item.SubItems[4].Text = "Failed";
                    item.UseItemStyleForSubItems = false;
                    item.SubItems[4].ForeColor = Color.Red;
                }
            }
        }

        private void Ping_All_Proxies()
        {
            foreach (ListViewItem item in listView1.Items)
            {
                try
                {
                    Ping pingIP = new Ping();
                    PingReply pingReply = pingIP.Send(item.Text);
                    if (pingReply.Status == IPStatus.Success)
                    {
                        item.SubItems[3].Text = pingReply.RoundtripTime.ToString();
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Green;
                    }
                    else
                    {
                        item.SubItems[3].Text = "";
                        item.SubItems[4].Text = pingReply.Status.ToString();
                        item.UseItemStyleForSubItems = false;
                        item.SubItems[4].ForeColor = Color.Red;
                    }
                }
                catch (Exception)
                {
                    MessageBox.Show("Could not ping \"" + item.Text + "\"\nMake sure you've typed IP address correctly! \n[XXX.XXX.XXX.XXX]", "Error pinging " + item.Text, MessageBoxButtons.OK, MessageBoxIcon.Error);
                    item.SubItems[3].Text = "";
                    item.SubItems[4].Text = "Failed";
                    item.UseItemStyleForSubItems = false;
                    item.SubItems[4].ForeColor = Color.Red;
                }
            }
        }
        #endregion

        #region Apply Proxy In Registry
        private void ApplyChanges()
        {
            ListView.CheckedListViewItemCollection checkedItem = listView1.CheckedItems;
            RegistryKey SystemProxy = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Internet Settings");

            try
            {
                if (listView1.CheckedItems.Count > 0)
                {
                    foreach (ListViewItem item in listView1.CheckedItems)
                    {
                        string ProxyPort = item.SubItems[0].Text + ":" + item.SubItems[1].Text;
                        SystemProxy.SetValue("ProxyServer", ProxyPort);
                        SystemProxy.SetValue("ProxyEnable", "1", RegistryValueKind.DWord);
                    }
                }
                else
                {
                    try
                    {
                        SystemProxy.SetValue("ProxyEnable", "0", RegistryValueKind.DWord);
                        SystemProxy.SetValue("ProxyServer", "");
                    }
                    catch (Exception exc)
                    {
                        MessageBox.Show(exc.Message);
                    }
                }
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message);
            }
        }
        #endregion

        #region Disabling Proxy
        private void DisableSystemProxy()
        {
            RegistryKey SystemProxy = Registry.CurrentUser.CreateSubKey(@"Software\Microsoft\Windows\CurrentVersion\Internet Settings");

            try
            {
                SystemProxy.SetValue("ProxyServer", "");
                SystemProxy.SetValue("ProxyEnable", "0", RegistryValueKind.DWord);
            }
            catch (Exception exc)
            {
                MessageBox.Show(exc.Message);
            }

            foreach (ListViewItem item in listView1.Items)
            {
                item.Checked = false;
            }
        }
        #endregion

        #region Window Buttons Behavior
        private void Add_button_Click(object sender, EventArgs e)
        {
            AddProxiesToListView();
            CreateXMLFile();
            WriteListViewToXML();
        }

        private void Help_button_Click(object sender, EventArgs e)
        {
            MessageBox.Show("This application helps you to change system proxy easily!\n\nartman_12@hotmail.com \n2012", "Help", MessageBoxButtons.OK, MessageBoxIcon.Information);
        }

        private void Test_Proxy_button_Click(object sender, EventArgs e)
        {
            Ping_All_Proxies();
        }

        private void Clear_button_Click(object sender, EventArgs e)
        {
            DisableSystemProxy();
        }

        private void Apply_button_Click(object sender, EventArgs e)
        {
            ApplyChanges();
            Ping_Current_Proxy();
        }
        #endregion

        #region Context Menu
        private void pingToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Ping_Selected_Proxy();
        }

        private void editCommentToolStripMenuItem_Click(object sender, EventArgs e)
        {
            foreach (ListViewItem item in listView1.SelectedItems)
            {
                pulledComment = item.SubItems[5].Text;
                using (dlgEditComment dialog = new dlgEditComment(pulledComment))
                {
                    if (dialog.ShowDialog(this) == DialogResult.OK)
                    {
                        item.SubItems[5].Text = dialog.newComment;
                        pulledComment = item.SubItems[5].Text;
                        CreateXMLFile();
                        WriteListViewToXML();
                    }
                }
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e)
        {
            DeleteItem();
        }
        #endregion
    }
}
